package org.xbib.event.async.impl;

import org.xbib.event.async.Async;
import org.xbib.event.async.Future;
import org.xbib.event.async.Handler;
import org.xbib.event.async.Promise;
import org.xbib.event.async.impl.future.PromiseInternal;

/**
 * This interface provides services for internal use only.
 * It is not part of the public API and should not be used by
 * developers.
 */
public interface AsyncInternal extends Async {

  /**
   * @return a promise associated with the context returned by {@link #getOrCreateContext()}.
   */
  <T> PromiseInternal<T> promise();

  /**
   * @return a promise associated with the context returned by {@link #getOrCreateContext()} or the {@code handler}
   *         if that handler is already an instance of {@code PromiseInternal}
   */
  <T> PromiseInternal<T> promise(Promise<T> promise);

  @Override
  ContextInternal getOrCreateContext();

  WorkerPool getWorkerPool();

  WorkerPool getInternalWorkerPool();

  /**
   * Get the current context
   * @return the context
   */
  ContextInternal getContext();

  default <T> Future<T> executeBlockingInternal(Handler<Promise<T>> blockingCodeHandler) {
    ContextInternal context = getOrCreateContext();
    return context.executeBlockingInternal(blockingCodeHandler);
  }

  default <T> Future<T> executeBlockingInternal(Handler<Promise<T>> blockingCodeHandler, boolean ordered) {
    ContextInternal context = getOrCreateContext();
    return context.executeBlockingInternal(blockingCodeHandler, ordered);
  }

  CloseFuture closeFuture();

}
