package org.xbib.event.async.yield.primitives.intgr.ops;

import org.xbib.event.async.yield.Yield;
import org.xbib.event.async.yield.primitives.intgr.IntAdvancer;
import org.xbib.event.async.yield.primitives.intgr.IntQuery;
import org.xbib.event.async.yield.primitives.intgr.IntTraverser;
import org.xbib.event.async.yield.primitives.intgr.IntYield;

import java.util.function.IntPredicate;

public class IntTakeWhile implements IntAdvancer, IntTraverser {

    private final IntQuery upstream;

    private final IntPredicate predicate;

    private boolean hasNext;

    public IntTakeWhile(IntQuery upstream, IntPredicate predicate) {
        this.upstream = upstream;
        this.predicate = predicate;
        this.hasNext = true;
    }

    @Override
    public void traverse(IntYield yield) {
        upstream.shortCircuit(item -> {
            if (!predicate.test(item)) {
                Yield.bye();
            }
            yield.ret(item);
        });
    }

    @Override
    public boolean tryAdvance(IntYield yield) {
        if (!hasNext) return false; // Once predicate is false it finishes the iteration
        IntYield takeWhile = item -> {
            if (predicate.test(item)) {
                yield.ret(item);
            } else {
                hasNext = false;
            }
        };
        return upstream.tryAdvance(takeWhile) && hasNext;
    }
}
