package org.xbib.event.clock;

import org.xbib.event.Event;
import org.xbib.event.common.AbstractEventManagerService;
import org.xbib.event.common.EventManager;
import org.xbib.event.common.EventManagerService;
import org.xbib.settings.Settings;
import org.xbib.time.schedule.CronExpression;
import org.xbib.time.schedule.CronSchedule;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ClockEventManagerService extends AbstractEventManagerService implements EventManagerService {

    private static final Logger logger = Logger.getLogger(ClockEventManagerService.class.getName());

    private EventManager eventManager;

    private CronSchedule<Boolean> cronSchedule;

    public ClockEventManagerService() {
        super();
    }

    @Override
    public ClockEventManagerService init(EventManager eventManager) {
        this.eventManager = eventManager;
        ThreadFactory threadFactory = new ThreadFactory() {
            int n = 1;
            @Override
            public Thread newThread(Runnable r) {
                return new Thread(r, "clock-event-manager-" + (n++));
            }
        };
        ScheduledExecutorService scheduledExecutorService =
                Executors.newScheduledThreadPool(eventManager.getSettings().getAsInt("pool.size", 2), threadFactory);
        this.cronSchedule = new CronSchedule<>(scheduledExecutorService);
        for (Map.Entry<String, Settings> mapEntry : eventManager.getSettings().getGroups("event.clock").entrySet()) {
            String name = mapEntry.getKey();
            Settings entrySettings = mapEntry.getValue();
            if (entrySettings.getAsBoolean("enabled", true)) {
                String entry = entrySettings.get("entry");
                String type = entrySettings.get("type", "clock");
                if (entry != null && type != null) {
                    try {
                        ClockEventService clockEventService = new ClockEventService(this, name, type);
                        cronSchedule.add(name, CronExpression.parse(entry), clockEventService);
                        logger.log(Level.INFO, "cron job " + name + "with type " + type + " scheduled on " + entry);
                    } catch (Exception e) {
                        logger.log(Level.WARNING, "unable to schedule cron job " + mapEntry.getKey() + ", reason " + e.getMessage());
                    }
                } else {
                    logger.log(Level.WARNING, "clock event service in configuration is incompletely defined, name = " + name );
                }
            } else {
                logger.log(Level.WARNING, "clock event service in configuration not enabled, name = " + name );
            }
        }
        cronSchedule.start();
        logger.log(Level.INFO, "after init: entries = " + cronSchedule.getEntries());
        return this;
    }

    @Override
    public void shutdown() throws IOException {
        cronSchedule.close();
    }

    public void publish(Event event) {
        eventManager.publish(event);
    }
}
