package org.xbib.event.log;

import java.nio.file.Path;
import java.text.MessageFormat;
import java.time.Instant;
import java.util.logging.Level;
import java.util.logging.Logger;

public class LoggingRotationListener implements RotationListener {

    private static final LoggingRotationListener INSTANCE = new LoggingRotationListener();

    private static final Logger logger = Logger.getLogger(LoggingRotationListener.class.getName());

    private LoggingRotationListener() {
    }

    public static LoggingRotationListener getInstance() {
        return INSTANCE;
    }

    @Override
    public void onTrigger(RotationPolicy policy, Instant instant) {
        logger.log(Level.FINE, MessageFormat.format("rotation trigger policy={0}, instant={1}", policy, instant));
    }

    @Override
    public void onOpen(RotationPolicy policy, Instant instant) {
        logger.log(Level.FINE, MessageFormat.format("open policy={0}, instant={1}", policy, instant));
    }

    @Override
    public void onClose(RotationPolicy policy, Instant instant) {
        logger.log(Level.FINE, MessageFormat.format("close policy={0}, instant={1}", policy, instant));
    }

    @Override
    public void onSuccess(RotationPolicy policy, Instant instant, Path path) {
        logger.log(Level.FINE, MessageFormat.format("rotation success policy={0}, instant={1}, path={2}", policy, instant, path));
    }

    @Override
    public void onFailure(RotationPolicy policy, Instant instant, Path path, Throwable throwable) {
        String message = MessageFormat.format("rotation failure policy={0}, instant={1}, file={2}", policy, instant, path);
        logger.log(Level.SEVERE, message, throwable);
    }
}
