package org.xbib.event.log;

import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;

public class SystemClock implements Clock {

    private static final SystemClock INSTANCE = new SystemClock();

    public SystemClock() {
    }

    public static SystemClock getInstance() {
        return INSTANCE;
    }

    @Override
    public Instant now() {
        return Instant.now();
    }

    @Override
    public Instant midnight() {
        Instant instant = now();
        ZonedDateTime utcInstant = instant.atZone(ZoneId.of("UTC"));
        return LocalDate
                .from(utcInstant)
                .atStartOfDay()
                .plusDays(1)
                .toInstant(ZoneOffset.UTC);
    }

    @Override
    public Instant sundayMidnight() {
        Instant instant = now();
        ZonedDateTime utcInstant = instant.atZone(ZoneId.of("UTC"));
        LocalDateTime todayStart = LocalDate.from(utcInstant).atStartOfDay();
        int todayIndex = todayStart.getDayOfWeek().getValue() - 1;
        int sundayOffset = 7 - todayIndex;
        return todayStart
                .plusDays(sundayOffset)
                .toInstant(ZoneOffset.UTC);
    }
}
