package org.xbib.event.wal;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.function.Supplier;

public class BlockingQueuePool<T> implements ObjectPool<T> {
    private final BlockingQueue<T> queue;
    private final Supplier<T> creationFunction;
    private final Predicate<T> reuseCheck;
    private final Consumer<T> returnPreparation;

    public BlockingQueuePool(final int maxSize, final Supplier<T> creationFunction, final Predicate<T> reuseCheck, final Consumer<T> returnPreparation) {
        this.queue = new LinkedBlockingQueue<>(maxSize);
        this.creationFunction = creationFunction;
        this.reuseCheck = reuseCheck;
        this.returnPreparation = returnPreparation;
    }

    @Override
    public T borrowObject() {
        final T existing = queue.poll();
        if (existing != null) {
            return existing;
        }

        return creationFunction.get();
    }

    @Override
    public void returnObject(final T somethingBorrowed) {
        if (reuseCheck.test(somethingBorrowed)) {
            returnPreparation.accept(somethingBorrowed);
            queue.offer(somethingBorrowed);
        }
    }
}
