package org.xbib.event.wal;

public interface SerDeFactory<T> {

    /**
     * Returns a new SerDe
     *
     * @param encodingName the name of encoding that was used when writing the serialized data, or <code>null</code> if
     *            the SerDe is to be used for serialization purposes
     * @return a SerDe
     */
    SerDe<T> createSerDe(String encodingName);

    /**
     * Returns the unique ID for the given record
     *
     * @param record to obtain identifier for
     * @return identifier of record
     */
    Object getRecordIdentifier(T record);

    /**
     * Returns the UpdateType for the given record
     *
     * @param record to retrieve update type for
     * @return update type
     */
    UpdateType getUpdateType(T record);

    /**
     * Returns the external location of the given record; this is used when a
     * record is moved away from WALI or is being re-introduced to WALI. For
     * example, WALI can be updated with a record of type
     * {@link UpdateType#SWAP_OUT} that indicates a Location of
     * file://tmp/external1 and can then be re-introduced to WALI by updating
     * WALI with a record of type {@link UpdateType#CREATE} that indicates a
     * Location of file://tmp/external1
     *
     * @param record to get location of
     * @return location
     */
    String getLocation(T record);
}
