package org.xbib.event.syslog;

import org.xbib.event.bus.EventBus;
import org.xbib.event.common.EventManager;
import org.xbib.event.common.EventManagerService;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SyslogEventManagerService implements EventManagerService {

    private static final Logger logger = Logger.getLogger(SyslogEventManagerService.class.getName());

    private List<SyslogService> syslogServices;

    public SyslogEventManagerService() {}

    @Override
    public SyslogEventManagerService init(EventManager eventManager) {
        Settings settings = eventManager.getSettings();
        EventBus eventBus = eventManager.getEventBus();
        this.syslogServices = new ArrayList<>();
        for (Map.Entry<String, Settings> entry : settings.getGroups("event.syslog").entrySet()) {
            Settings definition = entry.getValue();
            if (definition.getAsBoolean("enabled", true)) {
                try {
                    SyslogService syslogService = new SyslogService(definition, eventBus);
                    syslogServices.add(syslogService);
                    logger.log(Level.INFO, "syslog service " + entry.getKey() + " added");
                } catch (Exception e) {
                    logger.log(Level.SEVERE, "unable to create file follow service " + entry.getKey() + ", reason " + e.getMessage(), e);
                }
            }
        }
        return this;
    }

    @Override
    public void shutdown() throws IOException {
        for (SyslogService syslogService : syslogServices) {
            syslogService.close();
        }
    }
}
