package org.xbib.event.syslog;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.LineBasedFrameDecoder;
import io.netty.util.ByteProcessor;
import io.netty.util.CharsetUtil;

public class SyslogFrameDecoder extends LineBasedFrameDecoder {
    final static ByteProcessor INTEGER = b -> b >= ((byte) 48) && b <= ((byte) 57);

    public SyslogFrameDecoder(int maxLength) {
        super(maxLength, true, false);
    }

    @Override
    protected Object decode(ChannelHandlerContext ctx, ByteBuf b) throws Exception {
        ByteBuf buffer = b.retain();
        int lengthIndex = buffer.forEachByte(INTEGER);
        final int digitCount = lengthIndex - buffer.readerIndex();
        if (digitCount > 0) {
            buffer.markReaderIndex();
            final String frameLength = buffer.getCharSequence(buffer.readerIndex(), digitCount, CharsetUtil.UTF_8).toString();
            buffer.skipBytes(digitCount + 1);
            int length = Integer.parseInt(frameLength);

            if (b.readerIndex() + length > b.writerIndex()) {
                buffer.resetReaderIndex();
                return null;
            }
            return buffer.slice(digitCount + 1, length);
        } else {
            return super.decode(ctx, buffer);
        }
    }
}

