package org.xbib.event.syslog;

import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

@ChannelHandler.Sharable
public class SyslogMessageHandler extends SimpleChannelInboundHandler<SyslogRequest> {

    private final List<MessageParser> parsers;

    public SyslogMessageHandler() {
        this(Arrays.asList(new CEFMessageParser(), new RFC5424MessageParser(), new RFC3164MessageParser()));
    }

    public SyslogMessageHandler(List<MessageParser> parsers) {
        this.parsers = parsers;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext context, SyslogRequest request) {
        context.executor().submit(() -> {
            for (MessageParser parser : parsers) {
                try {
                    Object result = parser.parse(request);
                    if (result != null) {
                        context.fireChannelRead(result);
                        return;
                    }
                } catch (IOException e) {
                    throw new UncheckedIOException(e);
                }
            }
            context.write(DefaultSyslogMessage.builder()
                    .type(MessageType.UNKNOWN)
                    .date(LocalDateTime.now())
                    .rawMessage(request.rawMessage())
                    .remoteAddress(request.remoteAddress())
                    .build());
        });
    }
}
